@@ -37,6 +37,7 @@ use constant MSVC => WIN32() && ($Config{cc} eq 'cl');
 use constant REQUIRE_ITHREADS => grep { $^O eq $_ } qw(MSWin32);
 use constant PERL_HAS_ITHREADS =>
     $Config{useithreads} && ($Config{useithreads} eq 'define');
+use constant BUILD_APREXT => WIN32();
 
 use ModPerl::Code ();
 use ModPerl::BuildOptions ();
@@ -80,7 +81,7 @@ sub ap_prefix_invalid {
 
     my $include_dir = $self->apxs(-q => 'INCLUDEDIR');
 
-    unless (-e $include_dir) {
+    unless (-d $include_dir) {
         return "include/ directory not found in $prefix";
     }
 
@@ -98,22 +99,14 @@ sub httpd_is_source_tree {
         defined $prefix && -d $prefix && -e "$prefix/CHANGES";
 }
 
-sub apxs {
+# try to find the apxs utility, set $apxs to the path if found,
+# otherwise to ''
+my $apxs; # undef so we know we haven't tried to set it yet
+sub find_apxs_util {
     my $self = shift;
 
-    my $is_query = (@_ == 2) && ($_[0] eq '-q');
+    $apxs = ''; # not found
 
-    $self = $self->build_config unless ref $self;
-
-    my $query_key;
-    if ($is_query) {
-        $query_key = 'APXS_' . uc $_[1];
-        if ($self->{$query_key}) {
-            return $self->{$query_key};
-        }
-    }
-
-    my $apxs;
     my @trys = ($Apache::Build::APXS,
                 $self->{MP_APXS},
                 $ENV{MP_APXS},
@@ -136,13 +129,35 @@ sub apxs {
         '/usr/local/apache/bin/apxs';
     }
 
+    my $apxs_try;
     for (@trys) {
-        next unless ($apxs = $_);
-        chomp $apxs;
-        last if -x $apxs;
+        next unless ($apxs_try = $_);
+        chomp $apxs_try;
+        if (-x $apxs_try) {
+            $apxs = $apxs_try;
+            last;
+        }
+    }
+}
+
+sub apxs {
+    my $self = shift;
+
+    $self->find_apxs_util() unless defined $apxs;
+
+    my $is_query = (@_ == 2) && ($_[0] eq '-q');
+
+    $self = $self->build_config unless ref $self;
+
+    my $query_key;
+    if ($is_query) {
+        $query_key = 'APXS_' . uc $_[1];
+        if ($self->{$query_key}) {
+            return $self->{$query_key};
+        }
     }
 
-    unless ($apxs and -x $apxs) {
+    unless ($apxs) {
         my $prefix = $self->{MP_AP_PREFIX} || "";
         return '' unless -d $prefix and $is_query;
         my $val = $apxs_query{$_[1]};
@@ -151,16 +166,18 @@ sub apxs {
 
     my $devnull = devnull();
     my $val = qx($apxs @_ 2>$devnull);
-    chomp $val if defined $val; # apxs post-2.0.40 adds a new line
+    chomp $val if defined $val;
 
     unless ($val) {
-        error "'$apxs @_' failed:";
-
-        if (my $error = qx($apxs @_ 2>&1)) {
+        # do we have an error or is it just an empty value?
+        my $error = qx($apxs @_ 2>&1);
+        chomp $error if defined $error;
+        if ($error) {
+            error "'$apxs @_' failed:";
             error $error;
         }
         else {
-            error 'unknown error';
+            $val = '';
         }
     }
 
@@ -173,6 +190,18 @@ sub apxs_cflags {
     $cflags;
 }
 
+sub apxs_extra_cflags {
+    my $flags = __PACKAGE__->apxs('-q' => 'EXTRA_CFLAGS');
+    $flags =~ s/\"/\\\"/g;
+    $flags;
+}
+
+sub apxs_extra_cppflags {
+    my $flags = __PACKAGE__->apxs('-q' => 'EXTRA_CPPFLAGS');
+    $flags =~ s/\"/\\\"/g;
+    $flags;
+}
+
 my %threaded_mpms = map { $_ => 1}
         qw(worker winnt beos mpmt_os2 netware leader perchild threadpool);
 sub mpm_is_threaded {
@@ -216,6 +245,47 @@ sub mpm_name {
     return $self->{mpm_name} = $mpm_name;
 }
 
+sub should_build_apache {
+    my ($self) = @_;
+    return $self->{MP_AP_BUILD} ? 1 : 0;
+}
+
+sub configure_apache {
+    my ($self) = @_;
+
+    unless ($self->{MP_AP_CONFIGURE}) {
+        error "You specified MP_AP_BUILD but did not specify the " .
+              "arguments to httpd's ./configure with MP_AP_CONFIGURE";
+        exit 1;
+    }
+    
+    unless ($self->{MP_USE_STATIC}) {
+        error "When building httpd, you must set MP_USE_STATIC=1";
+        exit 1;
+    }
+
+    debug "Configuring httpd in $self->{MP_AP_PREFIX}";
+    
+    my $httpd = File::Spec->catfile($self->{MP_AP_PREFIX}, 'httpd');
+    push @Apache::TestMM::Argv, ('-httpd' => $httpd);
+    
+    my $mplib = "$self->{MP_LIBNAME}$Config{lib_ext}";
+    my $mplibpath = catfile($self->{cwd}, qw(src modules perl), $mplib);
+    
+    local $ENV{BUILTIN_LIBS} = $mplibpath;
+    local $ENV{AP_LIBS} = $self->ldopts;
+    local $ENV{MODLIST} = 'perl';
+
+    #XXX: -Wall and/or -Werror at httpd configure time breaks things
+    local $ENV{CFLAGS} = join ' ', grep { ! /\-Wall|\-Werror/ } 
+        split /\s+/, $ENV{CFLAGS};
+    
+    my $cd = qq(cd $self->{MP_AP_PREFIX});
+    my $cmd = qq(./configure $self->{MP_AP_CONFIGURE});
+    debug "Running $cmd";
+    system("$cd && $cmd") == 0 or die "httpd: $cmd failed";
+}
+
 #--- Perl Config stuff ---
 
 my %gtop_config = ();
@@ -240,6 +310,7 @@ sub find_gtop {
 sub find_gtop_config {
     my %c = ();
 
+    my $ver_2_5_plus = 0;
     if (system('pkg-config --exists libgtop-2.0') == 0) {
         # 2.x
         chomp($c{ccopts} = qx|pkg-config --cflags libgtop-2.0|);
@@ -248,6 +319,17 @@ sub find_gtop_config {
         # 2.0.0 bugfix
         chomp(my $libdir = qx|pkg-config --variable=libdir libgtop-2.0|);
         $c{ldopts} =~ s|\$\(libdir\)|$libdir|;
+
+        chomp($c{ver} = qx|pkg-config --modversion libgtop-2.0|);
+        ($c{ver_maj}, $c{ver_min}) = split /\./, $c{ver};
+        $ver_2_5_plus++ if $c{ver_maj} == 2 && $c{ver_min} >= 5;
+
+        if ($ver_2_5_plus) {
+            # some headers were removed in libgtop 2.5.0 so we need to
+            # be able to exclude them at compile time
+            $c{ccopts} .= ' -DGTOP_2_5_PLUS';
+        }
+
     }
     elsif (system('gnome-config --libs libgtop') == 0) {
         chomp($c{ccopts} = qx|gnome-config --cflags libgtop|);
@@ -258,7 +340,9 @@ sub find_gtop_config {
         $c{ldopts} =~ s|^/|-L/|;
     }
 
-    if ($c{ccopts}) {
+    # starting from 2.5.0 'pkg-config --cflags libgtop-2.0' already
+    # gives us all the cflags that are needed
+    if ($c{ccopts} && !$ver_2_5_plus) {
         chomp(my $ginc = `glib-config --cflags`);
         $c{ccopts} .= " $ginc";
     }
@@ -381,6 +465,13 @@ sub ap_ccopts {
         $ccopts .= " -DMP_TRACE";
     }
 
+    # make sure apr.h can be safely included
+    # for example Perl's included -D_GNU_SOURCE implies
+    # -D_LARGEFILE64_SOURCE on linux, but this won't happen on
+    # Solaris, so we need apr flags living in apxs' EXTRA_CPPFLAGS
+    my $extra_cppflags = $self->apxs_extra_cppflags;
+    $ccopts .= " " . $extra_cppflags;
+
     $ccopts;
 }
 
@@ -413,12 +504,26 @@ sub ccopts_hpux {
     $$cflags .= " -Ae ";
 }
 
+# XXX: there could be more, but this is just for cosmetics
+my %cflags_dups = map { $_ => 1 } qw(-D_GNU_SOURCE -D_REENTRANT);
 sub ccopts {
     my($self) = @_;
 
     my $cflags = $self->perl_ccopts . ExtUtils::Embed::perl_inc() .
                  $self->ap_ccopts;
 
+    # remove duplicates of certain cflags coming from perl and ap/apr
+    my @cflags = ();
+    my %dups    = ();
+    for (split /\s+/, $cflags) {
+        if ($cflags_dups{$_}) {
+            next if $dups{$_};
+            $dups{$_}++;
+        }
+        push @cflags, $_;
+    }
+    $cflags = "@cflags";
+
     $cflags;
 }
 
@@ -640,6 +745,8 @@ sub new {
         @_,
     }, $class;
 
+    $self->{MP_APR_LIB} = 'aprext';
+
     ModPerl::BuildOptions->init($self) if delete $self->{init};
 
     $self;
@@ -826,19 +933,52 @@ sub apr_bindir {
     $self->{apr_bindir};
 }
 
-# XXX: we assume that apr-config and apu-config reside in the same
-# directory
-sub apr_config_path {
-    my ($self) = @_;
+sub apr_generation {
+    my($self) = @_;
+    return $self->httpd_version_as_int =~ m/21\d+/ ? 1 : 0;
+}
 
-    return $self->{apr_config_path}
-        if $self->{apr_config_path} and -x $self->{apr_config_path};
+# returns an array of apr/apu linking flags (--link-ld --libs) if found
+# an empty array otherwise
+my @apru_link_flags = ();
+sub apru_link_flags {
+    my($self) = @_;
 
-    if (exists $self->{MP_APR_CONFIG} and -x $self->{MP_APR_CONFIG}) {
-        $self->{apr_config_path} = $self->{MP_APR_CONFIG};
+    return @apru_link_flags if @apru_link_flags;
+
+    for ($self->apr_config_path, $self->apu_config_path) {
+        if (my $link = $_ && -x $_ && qx{$_ --link-ld --libs}) {
+            chomp $link;
+            push @apru_link_flags, $link;
+        }
     }
 
-    if (!$self->{apr_config_path}) {
+    return @apru_link_flags;
+}
+
+sub apr_config_path {
+    shift->apru_config_path("apr");
+}
+
+sub apu_config_path {
+    shift->apru_config_path("apu");
+}
+
+sub apru_config_path {
+    my ($self, $what) = @_;
+
+    my $key = "${what}_config_path"; # apr_config_path
+    my $mp_key = "MP_" . uc($what) . "_CONFIG"; # MP_APR_CONFIG
+
+    return $self->{$key} if $self->{$key} and -x $self->{$key};
+
+    if (exists $self->{$mp_key} and -x $self->{$mp_key}) {
+        $self->{$key} = $self->{$mp_key};
+    }
+
+    my $config = $self->apr_generation ? "$what-1-config" : "$what-config";
+
+    if (!$self->{$key}) {
         my @tries = ();
         if ($self->httpd_is_source_tree) {
             push @tries, grep { -d $_ }
@@ -846,14 +986,13 @@ sub apr_config_path {
                 grep defined $_, $self->dir;
         }
         else {
-            # APR_BINDIR was added only at httpd-2.0.46
             push @tries, grep length,
                 map $self->apxs(-q => $_), qw(APR_BINDIR BINDIR);
             push @tries, catdir $self->{MP_AP_PREFIX}, "bin"
                 if exists $self->{MP_AP_PREFIX} and -d $self->{MP_AP_PREFIX};
         }
 
-        @tries = map { catfile $_, "apr-config" } @tries;
+        @tries = map { catfile $_, $config } @tries;
         if (WIN32) {
             my $ext = '.bat';
             for (@tries) {
@@ -863,22 +1002,22 @@ sub apr_config_path {
 
         for my $try (@tries) {
             next unless -x $try;
-            $self->{apr_config_path} = $try;
+            $self->{$key} = $try;
         }
     }
 
-    $self->{apr_config_path} ||= Apache::TestConfig::which('apr-config');
+    $self->{$key} ||= Apache::TestConfig::which($config);
 
     # apr_bindir makes sense only if httpd/apr is installed, if we are
     # building against the source tree we can't link against
     # apr/aprutil libs
     unless ($self->httpd_is_source_tree) {
-        $self->{apr_bindir} = $self->{apr_config_path}
-            ? dirname $self->{apr_config_path}
+        $self->{apr_bindir} = $self->{$key}
+            ? dirname $self->{$key}
             : '';
         }
 
-    $self->{apr_config_path};
+    $self->{$key};
 }
 
 sub apr_includedir {
@@ -1286,6 +1425,36 @@ sub modperl_libs {
     $libs->($self);
 }
 
+# returns the directory and name of the aprext lib built under blib/ 
+sub mp_apr_blib {
+    my $self = shift;
+    return unless (my $mp_apr_lib = $self->{MP_APR_LIB});
+    my $lib_mp_apr_lib = 'lib' . $mp_apr_lib;
+    my @dirs = $self->{MP_INST_APACHE2} ?
+        qw(blib arch Apache2 auto) : qw(blib arch auto);
+    my $apr_blib = catdir $self->{cwd}, @dirs, $lib_mp_apr_lib;
+    my $full_libname = $lib_mp_apr_lib . $Config{lib_ext};
+    return ($apr_blib, $full_libname);
+}
+
+sub mp_apr_lib_MSWin32 {
+    my $self = shift;
+    # The MP_APR_LIB will be installed into MP_AP_PREFIX/lib
+    # for use by 3rd party xs modules
+    my ($dir, $lib) = $self->mp_apr_blib();
+    $lib =~ s[^lib(\w+)$Config{lib_ext}$][$1];
+    $dir = Win32::GetShortPathName($dir);
+    return qq{ -L$dir -l$lib };
+}
+
+# linking used for the aprext lib used to build APR/APR::*
+sub mp_apr_lib {
+    my $self = shift;
+    my $libs = \&{"mp_apr_lib_$^O"};
+    return "" unless defined &$libs;
+    $libs->($self);
+}
+
 sub modperl_symbols_MSWin32 {
     my $self = shift;
     return "" unless $self->{MP_DEBUG};
@@ -1530,12 +1699,14 @@ sub includes {
         my $ap_inc = $self->apxs('-q' => 'INCLUDEDIR');
         if ($ap_inc && -d $ap_inc) {
             push @inc, $ap_inc;
-        } else {
-            # this is fatal
-            die "Can't find the mod_perl include dir";
+            return \@inc;
         }
 
-        return \@inc;
+        # this is fatal
+        my $reason = $ap_inc
+            ? "path $ap_inc doesn't exist"
+            : "apxs -q INCLUDEDIR didn't return a value";
+        die "Can't find the mod_perl include dir (reason: $reason)";
     }
 
     my $src = $self->dir;
@@ -1570,16 +1741,77 @@ sub inc {
     "@includes";
 }
 
+### Picking the right LFS support flags for mod_perl, by Joe Orton ###
+#
+# on Unix systems where by default off_t is a "long", a 32-bit integer,
+# there are two different ways to get "large file" support, i.e. the
+# ability to manipulate files bigger than 2Gb:
+#
+# 1) you compile using -D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64.  This
+# makes sys/types.h expose off_t as a "long long", a 64-bit integer, and
+# changes the size of a few other types too.  The C library headers
+# automatically arrange to expose a correct implementation of functions
+# like lseek() which take off_t parameters.
+#
+# 2) you compile using -D_LARGEFILE64_SOURCE, and use what is called the
+# "transitional" interface.  This means that the system headers expose a
+# new type, "off64_t", which is a long long, but the size of off_t is not
+# changed.   A bunch of new functions like lseek64() are exposed by the C 
+# library headers, which take off64_t parameters in place of off_t.
+#
+# Perl built with -Duselargefiles uses approach (1).
+#
+# APR HEAD uses (2) by default. APR 0.9 does not by default use either
+# approach, but random users can take a httpd-2.0.49 tarball, and do:
+#
+#   export CPPFLAGS="-D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64"
+#   ./configure
+#
+# to build a copy of apr/httpd which uses approach (1), though this
+# isn't really a supported configuration.
+#
+# The problem that mod_perl has to work around is when you take a
+# package built with approach (1), i.e. Perl, and any package which was
+# *not* built with (1), i.e. APR, and want to interface between
+# them. [1]
+#
+# So what you want to know is whether APR was built using approach (1)
+# or not.  APR_HAS_LARGE_FILES in HEAD just tells you whether APR was
+# built using approach (2) or not, which isn't useful in solving this
+# problem.
+#
+# [1]: In some cases, it may be OK to interface between packages which
+# use (1) and packages which use (2).  APR HEAD is currently not such a
+# case, since the size of apr_ino_t is still changing when
+# _FILE_OFFSET_BITS is defined.
+#
+# If you want to see how this matters, get some httpd function to do at
+# the very beginning of main():
+#
+#   printf("sizeof(request_rec) = %lu, sizeof(apr_finfo_t) = %ul",
+#          sizeof(request_rec), sizeof(apr_finfo_t));
+#
+# and then put the same printf in mod_perl somewhere, and see the
+# differences. This is why it is a really terribly silly idea to ever
+# use approach (1) in anything other than an entirely self-contained
+# application.
+#
 # there is no conflict if both libraries either have or don't have
 # large files support enabled
 sub has_large_files_conflict {
     my $self = shift;
-    my $apr_config = $self->get_apr_config();
 
-    my $perl = $Config{uselargefiles} ? 1 : 0;
-    my $apr  = $apr_config->{HAS_LARGE_FILES} ? 1 : 0;
+    my $apxs_flags = join $self->apxs_extra_cflags, $self->apxs_extra_cppflags;
+    my $apr_lfs64  = $apxs_flags      =~ /-D_FILE_OFFSET_BITS=64/;
+    my $perl_lfs64 = $Config{ccflags} =~ /-D_FILE_OFFSET_BITS=64/;
 
-    return $perl ^ $apr;
+    # XXX: we don't really deal with the case where APR was built with
+    # -D_FILE_OFFSET_BITS=64 but perl wasn't, since currently we strip
+    # only perl's ccflags, not apr's flags. the reason we don't deal
+    # with it is that we didn't have such a case yet, but may need to
+    # deal with it later
+
+    return $perl_lfs64 ^ $apr_lfs64;
 }
 
 # if perl is built with uselargefiles, but apr not, the build won't
