@@ -1,3 +1,17 @@
+# Copyright 2002-2004 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package ModPerl::Manifest;
 
 use strict;
@@ -13,10 +27,11 @@ our @EXPORT_OK = qw(mkmanifest);
 *import = \&Exporter::import;
 
 #generate a MANIFEST based on CVS/Entries
-#anythin to be skipped goes after __DATA__ (MANIFEST.SKIP format)
+#anything to be skipped goes after __DATA__ (MANIFEST.SKIP format)
 #anything else to be added should go here:
 my @add_files = qw{
     MANIFEST
+    Apache-Test/META.yml
 };
 
 sub get_cvs_files {
@@ -90,11 +105,10 @@ __DATA__
 patches/
 #very few will have Chatbot::Eliza installed
 eliza
-# fails with blead perl w/o useithreads
-t/response/TestAPR/perlio.pm
-# segfaults with blead perl
-t/filter/TestFilter/out_str_req_mix.pm
-t/filter/out_str_req_mix.t
+# random failures
+t/perl/ithreads.t
+t/perl/ithreads2.t
+t/response/TestPerl/ithreads.pm
 # incomplete
-t/error/push_handlers.t
-t/response/TestError/push_handlers.pm
+t/apr-ext/perlio
+t/modperl/merge3
