@@ -21,11 +21,14 @@
 #endif
 
 #ifdef MP_USE_GTOP
-
+/* prevent undef symbol errors (glibtop_error_vr) */
+#define __GLIBTOP_ERROR_H__
 #include <glibtop.h>
 #include <glibtop/open.h>
 #include <glibtop/close.h>
+#ifndef GTOP_2_5_PLUS
 #include <glibtop/xmalloc.h>
+#endif
 #include <glibtop/parameter.h>
 #include <glibtop/union.h>
 #include <glibtop/sysdeps.h>
