@@ -179,107 +179,16 @@ MP_INLINE SV *modperl_ptr2obj(pTHX_ char *classname, void *ptr)
     return sv;
 }
 
-/* XXX: sv_setref_uv does not exist in 5.6.x */
-MP_INLINE SV *modperl_perl_sv_setref_uv(pTHX_ SV *rv,
-                                        const char *classname, UV uv)
-{
-    sv_setuv(newSVrv(rv, classname), uv);
-    return rv;
-}
-
-static apr_pool_t *modperl_sv2pool(pTHX_ SV *obj, CV *method)
-{
-    apr_pool_t *p = NULL;
-    char *classname = NULL;
-    IV ptr = 0;
-
-    if ((SvROK(obj) && (SvTYPE(SvRV(obj)) == SVt_PVMG))) {
-        /* standard classes */
-        classname = SvCLASS(obj);
-        ptr = SvObjIV(obj);
-    }
-    else if ((SvROK(obj) && (SvTYPE(SvRV(obj)) == SVt_PVHV))) {
-        /* Apache::RequestRec subclass */
-        classname = SvCLASS(obj);
-        ptr = SvIV(modperl_hv_request_find(aTHX_ obj, classname, method));
-
-        /* if modperl_hv_request_find succeeeds then the class is an 
-         * Apache::RequestRec subclass (the only subclass we support).
-         * so, fake things a bit so we can dig out the proper pool below
-         */
-         classname = "Apache::RequestRec";
-    }
-    else {
-        MP_TRACE_m(MP_FUNC, "SV not a recognized object");
-        return NULL;
-    }
-
-    if (strnEQ(classname, "APR::", 5)) {
-        classname += 5;
-        switch (*classname) {
-          case 'P':
-            if (strEQ(classname, "Pool")) {
-                p = (apr_pool_t *)SvObjIV(obj);
-            }
-            break;
-          default:
-            MP_TRACE_m(MP_FUNC, "class %s not recognized", classname);
-            break;
-        };
-    }
-    else if (strnEQ(classname, "Apache::", 8)) {
-        classname += 8;
-        switch (*classname) {
-          case 'C':
-            if (strEQ(classname, "Connection")) {
-                p = ((conn_rec *)ptr)->pool;
-            }
-            break;
-          case 'R':
-            if (strEQ(classname, "RequestRec")) {
-                p = ((request_rec *)ptr)->pool;
-            }
-            break;
-          case 'S':
-            if (strEQ(classname, "Server")) {
-                p = ((server_rec *)ptr)->process->pconf;
-            }
-            break;
-          default:
-            MP_TRACE_m(MP_FUNC, "class %s not recognised", classname);
-            break;
-        };
-    }
-    else {
-        MP_TRACE_m(MP_FUNC, "class %s not recognised", classname);
-    }
-
-    if (p == NULL) {
-        MP_TRACE_m(MP_FUNC, "unable to derive pool from object");
-    }
-
-    return p;
-}
-
-char *modperl_apr_strerror(apr_status_t rv)
-{
-    dTHX;
-    char buf[256];
-    apr_strerror(rv, buf, sizeof(buf));
-    return Perl_form(aTHX_ "%d:%s", rv, buf);
-}
-
 int modperl_errsv(pTHX_ int status, request_rec *r, server_rec *s)
 {
     SV *sv = ERRSV;
     STRLEN n_a;
 
     if (SvTRUE(sv)) {
-        if (SvMAGICAL(sv) && (SvCUR(sv) > 4) &&
-            strnEQ(SvPVX(sv), " at ", 4))
-        {
+        if (sv_derived_from(sv, "APR::Error") &&
+            SvIVx(sv) == MODPERL_RC_EXIT) {
             /* ModPerl::Util::exit was called */
-            return DECLINED;
+            return OK;
         }
 #if 0
         if (modperl_sv_is_http_code(ERRSV, &status)) {
@@ -299,26 +208,22 @@ int modperl_errsv(pTHX_ int status, request_rec *r, server_rec *s)
     return status;
 }
 
-char *modperl_server_desc(server_rec *s, apr_pool_t *p)
+/* prepends the passed sprintf-like arguments to ERRSV, which also
+ * gets stringified on the way */
+void modperl_errsv_prepend(pTHX_ const char *pat, ...)
 {
-    return apr_psprintf(p, "%s:%u", s->server_hostname, s->port);
+    SV *sv;
+    va_list args;
+
+    va_start(args, pat);
+    sv = vnewSVpvf(pat, &args);
+    va_end(args);
+
+    sv_catsv(sv, ERRSV);
+    sv_copypv(ERRSV, sv);
+    sv_free(sv);
 }
 
-/* used in debug traces */
-MP_INLINE char *modperl_pid_tid(apr_pool_t *p)
-{
-    return apr_psprintf(p, "%lu"
-#if APR_HAS_THREADS
-                 "/%lu"
-#endif /* APR_HAS_THREADS */
-                 , (unsigned long)getpid()
-#if APR_HAS_THREADS
-                 , (unsigned long)apr_os_thread_current()
-#endif /* APR_HAS_THREADS */
-        );
-}
-
-    
 #define dl_librefs "DynaLoader::dl_librefs"
 #define dl_modules "DynaLoader::dl_modules"
 
@@ -338,7 +243,7 @@ void **modperl_xs_dl_handles_get(pTHX)
     void **handles;
 
     if (!librefs) {
-	MP_TRACE_g(MP_FUNC,
+	MP_TRACE_r(MP_FUNC,
                    "Could not get @%s for unloading.\n",
                    dl_librefs);
 	return NULL;
@@ -357,14 +262,14 @@ void **modperl_xs_dl_handles_get(pTHX)
 	SV *module_sv = *av_fetch(modules, i, FALSE);
 
 	if(!handle_sv) {
-	    MP_TRACE_g(MP_FUNC,
+	    MP_TRACE_r(MP_FUNC,
                        "Could not fetch $%s[%d]!\n",
                        dl_librefs, (int)i);
 	    continue;
 	}
 	handle = (void *)SvIV(handle_sv);
 
-	MP_TRACE_g(MP_FUNC, "%s dl handle == 0x%lx\n",
+	MP_TRACE_r(MP_FUNC, "%s dl handle == 0x%lx\n",
                    SvPVX(module_sv), (unsigned long)handle);
 	if (handle) {
 	    handles[i] = handle;
@@ -388,7 +293,7 @@ void modperl_xs_dl_handles_close(void **handles)
     }
 
     for (i=0; handles[i]; i++) {
-        MP_TRACE_g(MP_FUNC, "close 0x%lx\n", (unsigned long)handles[i]);
+        MP_TRACE_r(MP_FUNC, "close 0x%lx\n", (unsigned long)handles[i]);
         modperl_sys_dlclose(handles[i]);
     }
 
@@ -404,61 +309,6 @@ modperl_cleanup_data_t *modperl_cleanup_data_new(apr_pool_t *p, void *data)
     return cdata;
 }
 
-MP_INLINE modperl_uri_t *modperl_uri_new(apr_pool_t *p)
-{
-    modperl_uri_t *uri = (modperl_uri_t *)apr_pcalloc(p, sizeof(*uri));
-    uri->pool = p;
-    return uri;
-}
-
-MP_INLINE SV *modperl_hash_tie(pTHX_ 
-                               const char *classname,
-                               SV *tsv, void *p)
-{
-    SV *hv = (SV*)newHV();
-    SV *rsv = sv_newmortal();
-
-    sv_setref_pv(rsv, classname, p);
-    sv_magic(hv, rsv, PERL_MAGIC_tied, Nullch, 0);
-
-    return SvREFCNT_inc(sv_bless(sv_2mortal(newRV_noinc(hv)),
-                                 gv_stashpv(classname, TRUE)));
-}
-
-MP_INLINE void *modperl_hash_tied_object(pTHX_ 
-                                         const char *classname,
-                                         SV *tsv)
-{
-    if (sv_derived_from(tsv, classname)) {
-        if (SVt_PVHV == SvTYPE(SvRV(tsv))) {
-            SV *hv = SvRV(tsv);
-            MAGIC *mg;
-
-            if (SvMAGICAL(hv)) {
-                if ((mg = mg_find(hv, PERL_MAGIC_tied))) {
-                    return (void *)MgObjIV(mg);
-                }
-                else {
-                    Perl_warn(aTHX_ "Not a tied hash: (magic=%c)", mg);
-                }
-            }
-            else {
-                Perl_warn(aTHX_ "SV is not tied");
-            }
-        }
-        else {
-            return (void *)SvObjIV(tsv);
-        }
-    }
-    else {
-        Perl_croak(aTHX_
-                   "argument is not a blessed reference "
-                   "(expecting an %s derived object)", classname);
-    }
-
-    return NULL;
-}
-
 MP_INLINE void modperl_perl_av_push_elts_ref(pTHX_ AV *dst, AV *src)
 {
     I32 i, j, src_fill = AvFILLp(src), dst_fill = AvFILLp(dst);
@@ -544,6 +394,11 @@ void modperl_perl_call_list(pTHX_ AV *subs, const char *name)
 {
     I32 i, oldscope = PL_scopestack_ix;
     SV **ary = AvARRAY(subs);
+
+    MP_TRACE_g(MP_FUNC, "pid %lu" MP_TRACEf_TID MP_TRACEf_PERLID
+               " running %d %s subs",
+               (unsigned long)getpid(), MP_TRACEv_TID_ MP_TRACEv_PERLID_
+               AvFILLp(subs)+1, name);
     
     for (i=0; i<=AvFILLp(subs); i++) {
 	CV *cv = (CV*)ary[i];
@@ -565,15 +420,10 @@ void modperl_perl_call_list(pTHX_ AV *subs, const char *name)
 
 void modperl_perl_exit(pTHX_ int status)
 {
-    const char *pat = NULL;
     ENTER;
     SAVESPTR(PL_diehook);
     PL_diehook = Nullsv; 
-    sv_setpv(ERRSV, "");
-#ifdef MP_PERL_5_6_0
-    pat = ""; /* NULL segvs in 5.6.0 */
-#endif
-    Perl_croak(aTHX_ pat);
+    modperl_croak(aTHX_ MODPERL_RC_EXIT, "ModPerl::Util::exit");
 }
 
 MP_INLINE SV *modperl_dir_config(pTHX_ request_rec *r, server_rec *s,
@@ -641,16 +491,6 @@ MP_INLINE int modperl_perl_module_loaded(pTHX_ const char *name)
     return (*name && gv_stashpv(name, FALSE)) ? 1 : 0;
 }
 
-/* same as Symbol::gensym() */
-SV *modperl_perl_gensym(pTHX_ char *pack)
-{
-    GV *gv = newGVgen(pack);
-    SV *rv = newRV((SV*)gv);
-    (void)hv_delete(gv_stashpv(pack, TRUE), 
-                    GvNAME(gv), GvNAMELEN(gv), G_DISCARD);
-    return rv;
-}
-
 static int modperl_gvhv_is_stash(GV *gv)
 {
     int len = GvNAMELEN(gv);
@@ -709,7 +549,7 @@ void modperl_clear_symtab(pTHX_ HV *symtab)
     if (rc != APR_SUCCESS) { \
         SvREFCNT_dec(sv); \
         Perl_croak(aTHX_ "Error " action " '%s': %s ", r->filename, \
-                   modperl_apr_strerror(rc)); \
+                   modperl_error_strerror(aTHX_ rc)); \
     }
 
 MP_INLINE SV *modperl_slurp_filename(pTHX_ request_rec *r, int tainted)
@@ -764,27 +604,6 @@ MP_INLINE SV *modperl_slurp_filename(pTHX_ request_rec *r, int tainted)
     return newRV_noinc(sv);
 }
 
-#ifdef MP_TRACE
-/* XXX: internal debug function */
-/* any non-false value for MOD_PERL_TRACE/PerlTrace enables this function */
-void modperl_apr_table_dump(pTHX_ apr_table_t *table, char *name)
-{
-    int i;
-    const apr_array_header_t *array;
-    apr_table_entry_t *elts;
-
-    array = apr_table_elts(table);
-    elts  = (apr_table_entry_t *)array->elts;
-    modperl_trace(MP_FUNC, "Contents of table %s", name);
-    for (i = 0; i < array->nelts; i++) {
-        if (!elts[i].key || !elts[i].val) {
-            continue;
-        }
-        modperl_trace(MP_FUNC, "%s => %s", elts[i].key, elts[i].val);
-    }    
-}
-#endif
-
 #define MP_VALID_PKG_CHAR(c) (isalnum(c) ||(c) == '_')
 #define MP_VALID_PATH_DELIM(c) ((c) == '/' || (c) =='\\')
 char *modperl_file2package(apr_pool_t *p, const char *file)
@@ -835,26 +654,62 @@ char *modperl_file2package(apr_pool_t *p, const char *file)
     return package;
 }
 
-/* this is used across server_root_relative() in the
- * Apache, Apache::Server, Apache::RequestRec, and 
- * Apache::Connection classes
- */
-SV *modperl_server_root_relative(pTHX_ SV *sv, const char *fname)
+char *modperl_coderef2text(pTHX_ apr_pool_t *p, CV *cv)
 {
-    apr_pool_t *p;
+    dSP;
+    int count;
+    SV *bdeparse;
+    char *text;
+    
+    /* B::Deparse >= 0.61 needed for blessed code references.
+     * 0.6 works fine for non-blessed code refs.
+     * notice that B::Deparse is not CPAN-updatable.
+     * 0.61 is available starting from 5.8.0
+     */
+    Perl_load_module(aTHX_ PERL_LOADMOD_NOIMPORT,
+                     newSVpvn("B::Deparse", 10),
+                     newSVnv(SvOBJECT((SV*)cv) ? 0.61 : 0.60));
 
-    if (!sv_isobject(sv)) {
-        Perl_croak(aTHX_ "usage: Apache::server_root_relative(obj, name)");
+    ENTER;
+    SAVETMPS;
+
+    /* create the B::Deparse object */
+    PUSHMARK(sp);
+    XPUSHs(sv_2mortal(newSVpvn("B::Deparse", 10)));
+    PUTBACK;
+    count = call_method("new", G_SCALAR);
+    SPAGAIN;
+    if (count != 1) {
+        Perl_croak(aTHX_ "Unexpected return value from B::Deparse::new\n");
     }
-
-    p = modperl_sv2pool(aTHX_ sv, get_cv("Apache::server_root_relative", 0));
-
-    if (p == NULL) {
-        MP_TRACE_a(MP_FUNC,
-                   "unable to isolate pool for ap_server_root_relative()");
-        return &PL_sv_undef;
+    if (SvTRUE(ERRSV)) {
+        Perl_croak(aTHX_ "error: %s", SvPVX(ERRSV));
     }
+    bdeparse = POPs;
 
-    /* copy the SV in case the pool goes out of scope before the perl scalar */
-    return newSVpv(ap_server_root_relative(p, fname), 0);
+    PUSHMARK(sp);
+    XPUSHs(bdeparse);
+    XPUSHs(sv_2mortal(newRV_inc((SV*)cv)));
+    PUTBACK;
+    count = call_method("coderef2text", G_SCALAR);
+    SPAGAIN;
+    if (count != 1) {
+        Perl_croak(aTHX_ "Unexpected return value from "
+                   "B::Deparse::coderef2text\n");
+    }
+    if (SvTRUE(ERRSV)) {
+        Perl_croak(aTHX_ "error: %s", SvPVX(ERRSV));
+    }
+    
+    {
+        STRLEN n_a;
+        text = apr_pstrcat(p, "sub ", POPpx, NULL);
+    }
+    
+    PUTBACK;
+    
+    FREETMPS;
+    LEAVE;
+
+    return text;
 }
