@@ -3,7 +3,7 @@ package TestModperl::dir_config;
 use strict;
 use warnings FATAL => 'all';
 
-use Apache::Server ();
+use Apache::ServerRec ();
 use Apache::ServerUtil ();
 use Apache::RequestUtil ();
 use APR::Table ();
@@ -16,7 +16,7 @@ use Apache::Const -compile => 'OK';
 sub handler {
     my $r = shift;
 
-    plan $r, tests => 13;
+    plan $r, tests => 14;
 
     #Apache::RequestRec::dir_config tests
 
@@ -34,15 +34,15 @@ sub handler {
         my @received = $dir_config->get($key);
         my @expected = qw(1_SetValue 2_AddValue 3_AddValue 4_AddValue);
 
-        ok t_cmp(\@expected, \@received,
+        ok t_cmp(\@received, \@expected,
                  'PerlAddVar ITERATE2');
     }
 
     # sub-section inherits from super-section if it doesn't override it
     {
         my $key = make_key('_set_in_Base');
-        ok t_cmp('BaseValue',
-                 $r->dir_config($key),
+        ok t_cmp($r->dir_config($key),
+                 'BaseValue',
                  "sub-section inherits from super-section " .
                  "if it doesn't override it");
     }
@@ -50,7 +50,7 @@ sub handler {
     # sub-section overrides super-section for the same key
     {
         my $key = 'TestModperl__server_rec_Key_set_in_Base';
-        ok t_cmp('SubSecValue', $r->dir_config->get($key),
+        ok t_cmp($r->dir_config->get($key), 'SubSecValue',
                  "sub-section overrides super-section for the same key");
     }
 
@@ -59,23 +59,32 @@ sub handler {
 
         # object interface test in a scalar context (for a single
         # PerlSetVar key)
-        ok t_cmp('SetValue0',
-                 $dir_config->get($key),
+        ok t_cmp($dir_config->get($key),
+                 'SetValue0',
                  "table get() in a scalar context");
 
         # direct fetch test in a scalar context (for a single
         # PerlSetVar key)
-        ok t_cmp('SetValue0',
-                 $r->dir_config($key),
+        ok t_cmp($r->dir_config($key),
+                 'SetValue0',
                  "direct value fetch in a scalar context");
     }
 
+    # make sure 0 comes through as 0 and not undef
+    {
+        my $key = 'TestModperl__request_rec_ZeroKey';
+
+        ok t_cmp($r->dir_config($key),
+                 0,
+                 'table value 0 is not undef');
+    }
+
     # test non-existent key
     {
         my $key = make_key();
 
-        ok t_cmp(undef,
-                 $r->dir_config($key),
+        ok t_cmp($r->dir_config($key),
+                 undef,
                  "non-existent key");
     }
 
@@ -86,8 +95,8 @@ sub handler {
 
         $r->dir_config($key => $val);
 
-        ok t_cmp($val,
-                 $r->dir_config($key),
+        ok t_cmp($r->dir_config($key),
+                 $val,
                  "set && get");
     }
 
@@ -122,20 +131,20 @@ sub handler {
         my @received = $dir_config->get($key);
         my @expected = qw(1_SetValue 2_AddValue 3_AddValue);
 
-        ok t_cmp(\@expected, \@received,
+        ok t_cmp(\@received, \@expected,
                  "testing PerlAddVar ITERATE2 in \$s");
     }
 
     {
         # base server test
         my $bs = Apache->server;
-        ok t_cmp('Apache::Server',
-                 ($bs && ref($bs)),
+        ok t_cmp(($bs && ref($bs)),
+                 'Apache::ServerRec',
                  "base server's object retrieval");
 
         my $key = 'TestModperl__server_rec_Key_set_in_Base';
-        ok t_cmp('1_SetValue',
-                 scalar ($bs->dir_config->get($key)),
+        ok t_cmp(scalar ($bs->dir_config->get($key)),
+                 '1_SetValue',
                  "read dir_config of the base server");
     }
 
@@ -160,6 +169,8 @@ __END__
     PerlAddVar TestModperl__server_rec_Key_set_in_Base 2_AddValue 3_AddValue
 </Base>
 
+PerlSetVar TestModperl__request_rec_ZeroKey 0
+
 PerlSetVar TestModperl__request_rec_Key0 SetValue0
 
 PerlSetVar TestModperl__request_rec_Key1 ToBeLost
