@@ -155,6 +155,39 @@ static MP_INLINE SV *mpxs_apr_pool_create(pTHX_ SV *parent_pool_obj)
     }
 }
 
+static MP_INLINE void mpxs_APR__Pool_clear(pTHX_ SV *obj)
+{
+    apr_pool_t *p = mp_xs_sv2_APR__Pool(obj);
+    mpxs_pool_account_t *data;
+
+    apr_pool_userdata_get((void **)&data, MP_APR_POOL_NEW, p);
+    if (!(data && data->sv)) {
+        MP_POOL_TRACE(MP_FUNC, "parent pool (0x%lx) is a core pool",
+                      (unsigned long)p);
+        apr_pool_clear(p);
+        return;
+    }
+
+    MP_POOL_TRACE(MP_FUNC,
+                  "parent pool (0x%lx) is a custom pool, sv 0x%lx",
+                  (unsigned long)p,
+                  (unsigned long)data->sv);
+
+    apr_pool_clear(p);
+
+    /* apr_pool_clear removes all the user data, so we need to restore
+     * it. Since clear triggers mpxs_apr_pool_cleanup call, our
+     * object's guts get nuked too, so we need to restore them too */
+
+    /* this is sv_setref_pv, but for an existing object */
+    sv_setiv(newSVrv(obj, "APR::Pool"), PTR2IV((void*)p));
+    data->sv = SvRV(obj);
+
+    /* reinstall the user data */
+    apr_pool_userdata_set(data, MP_APR_POOL_NEW, NULL, p);
+}
+
+
 typedef struct {
     SV *cv;
     SV *arg;
@@ -169,10 +202,16 @@ typedef struct {
  * callback wrapper for Perl cleanup subroutines
  * @param data   internal storage
  */
+
+#ifndef MP_SOURCE_SCAN
+#include "apr_optional.h"
+static
+APR_OPTIONAL_FN_TYPE(modperl_interp_unselect) *modperl_opt_interp_unselect;
+#endif
+
 static apr_status_t mpxs_cleanup_run(void *data)
 {
     int count;
-    apr_status_t status = APR_SUCCESS;
     mpxs_cleanup_t *cdata = (mpxs_cleanup_t *)data;
     dTHXa(cdata->perl);
     dSP;
@@ -189,36 +228,37 @@ static apr_status_t mpxs_cleanup_run(void *data)
     SPAGAIN;
 
     if (count == 1) {
-        status = POPi;
+        (void)POPs; /* the return value is ignored */
     }
 
     PUTBACK;
     FREETMPS;LEAVE;
 
-    if (SvTRUE(ERRSV)) {
-        /*XXX*/
-    }
-
     SvREFCNT_dec(cdata->cv);
     if (cdata->arg) {
         SvREFCNT_dec(cdata->arg);
     }
 
 #ifdef USE_ITHREADS
-    if (cdata->interp) {
+    if (cdata->interp && modperl_opt_interp_unselect) {
         /* this will decrement the interp refcnt until
          * there are no more references, in which case
          * the interpreter will be putback into the mip
          */
-        (void)modperl_interp_unselect(cdata->interp);
+        (void)modperl_opt_interp_unselect(cdata->interp);
     }
 #endif
 
-    return status;
+    if (SvTRUE(ERRSV)) {
+        Perl_croak(aTHX_ SvPV_nolen(ERRSV));
+    }
+    
+    /* the return value is ignored by apr_pool_destroy anyway */
+    return APR_SUCCESS;
 }
 
 /**
- * run registered cleanups
+ * register cleanups to run
  * @param p      pool with which to associate the cleanup
  * @param cv     subroutine reference to run
  * @param arg    optional argument to pass to the subroutine
