@@ -13,8 +13,14 @@
  * limitations under the License.
  */
 
+#define mpxs_Apache__ServerRec_method_register(s, methname)    \
+    ap_method_register(s->process->pconf, methname);
+
+#define mpxs_Apache__ServerRec_add_version_component(s, component)    \
+    ap_add_version_component(s->process->pconf, component);
+
 static MP_INLINE
-int mpxs_Apache__Server_push_handlers(pTHX_ server_rec *s,
+int mpxs_Apache__ServerRec_push_handlers(pTHX_ server_rec *s,
                                       const char *name,
                                       SV *sv)
 {
@@ -27,7 +33,7 @@ int mpxs_Apache__Server_push_handlers(pTHX_ server_rec *s,
 }
 
 static MP_INLINE
-int mpxs_Apache__Server_set_handlers(pTHX_ server_rec *s,
+int mpxs_Apache__ServerRec_set_handlers(pTHX_ server_rec *s,
                                      const char *name,
                                      SV *sv)
 {
@@ -39,7 +45,7 @@ int mpxs_Apache__Server_set_handlers(pTHX_ server_rec *s,
 }
 
 static MP_INLINE
-SV *mpxs_Apache__Server_get_handlers(pTHX_ server_rec *s,
+SV *mpxs_Apache__ServerRec_get_handlers(pTHX_ server_rec *s,
                                      const char *name)
 {
     MpAV **handp =
@@ -51,20 +57,22 @@ SV *mpxs_Apache__Server_get_handlers(pTHX_ server_rec *s,
                                              s->process->pconf);
 }
 
-#define mpxs_Apache__Server_dir_config(s, key, sv_val) \
+#define mpxs_Apache__ServerRec_dir_config(s, key, sv_val) \
     modperl_dir_config(aTHX_ NULL, s, key, sv_val)
 
-#define mpxs_Apache_server(classname) \
-modperl_global_get_server_rec()
-
-#define mpxs_Apache__Server_server_root_relative(sv, fname) \
-    modperl_server_root_relative(aTHX_ sv, fname);
-
-#define mpxs_Apache_server_root_relative(sv, fname) \
-    modperl_server_root_relative(aTHX_ sv, fname);
+#define mpxs_Apache_server(classname) modperl_global_get_server_rec()
 
 static MP_INLINE
-int mpxs_Apache__Server_is_perl_option_enabled(pTHX_ server_rec *s,
+SV *mpxs_Apache__ServerUtil_server_root_relative(pTHX_ apr_pool_t *p,
+                                                 const char *fname)
+{
+    /* copy the SV in case the pool goes out of scope before the perl
+     * scalar */
+    return newSVpv(ap_server_root_relative(p, fname), 0);
+}
+
+static MP_INLINE
+int mpxs_Apache__ServerRec_is_perl_option_enabled(pTHX_ server_rec *s,
                                                const char *name)
 {
     return modperl_config_is_perl_option_enabled(aTHX_ NULL, s, name);
@@ -72,7 +80,7 @@ int mpxs_Apache__Server_is_perl_option_enabled(pTHX_ server_rec *s,
 
 
 static MP_INLINE
-void mpxs_Apache__Server_add_config(pTHX_ server_rec *s, SV *lines)
+void mpxs_Apache__ServerRec_add_config(pTHX_ server_rec *s, SV *lines)
 {
     const char *errmsg = modperl_config_insert_server(aTHX_ s, lines);
     if (errmsg) {
@@ -82,12 +90,12 @@ void mpxs_Apache__Server_add_config(pTHX_ server_rec *s, SV *lines)
 
 static void mpxs_Apache__ServerUtil_BOOT(pTHX)
 {
-    newCONSTSUB(PL_defstash, "Apache::server_root",
+    newCONSTSUB(PL_defstash, "Apache::ServerUtil::server_root",
                 newSVpv(ap_server_root, 0));
 
-    newCONSTSUB(PL_defstash, "Apache::get_server_built",
+    newCONSTSUB(PL_defstash, "Apache::ServerUtil::get_server_built",
                 newSVpv(ap_get_server_built(), 0));
 
-    newCONSTSUB(PL_defstash, "Apache::get_server_version",
+    newCONSTSUB(PL_defstash, "Apache::ServerUtil::get_server_version",
                 newSVpv(ap_get_server_version(), 0));
 }
